/*************************************************************************
 * Name        : Keith Vertanen
 * Username    : kvertanen
 * Description : My own version of a map that maps a String to a 
 *               String.  Note: this is a dumb and slow way to do it
 *               since it searches a linear list of pairs.
 *************************************************************************/

import java.util.*;

public class StringToStringDumb 
{	
	// Inner-class that stores an entry in our hash.
	// We need both the key and value stored so we can handle collisions in a bucket.
	private class Pair
	{
		String key;
		String value;
		
		public Pair(String key, String value)
		{
			this.key = key;
			this.value = value;
		}
	}

	// We hold all our pairs in a dynamically growing array
	ArrayList<Pair> pairs = new ArrayList<Pair>();
	
	// Add a new entry to the hash for the (key, value) pair
	public void put(String key, String value)
	{
		for (Pair pair : pairs)
		{
			// Look for an existing pair with the same key
			if (pair.key.equals(key))
			{
				pair.value = value;
				return;
			}			
		}
		// No existing entry has same value, so add a new one
		pairs.add(new Pair(key, value));
	}
	
	// Given a key, find the value stored in the hash (if any).
	// Return null if we didn't find this key.
	public String get(String key)
	{
		for (Pair pair : pairs)
		{
			// Look for a pair with a matching key
			if (pair.key.equals(key))
				return pair.value;
		}
		// We didn't find a matching key
		return null;
	}
	
	// Test main method
	public static void main(String [] args)
	{
		StringToStringHash map = new StringToStringHash();
		
		map.put("www.cnn.com"    , "157.166.255.19" );
		map.put("cs.mtech.edu"   , "150.131.202.136");		
		map.put("www.google.com" , "74.125.224.240" );
		map.put("google.com"     , "74.125.224.240" );
		map.put("www.cnn.com"    , "127.0.0.1" );
		
		// Let the use type a domain name, try and map to an IP
		while (!StdIn.isEmpty())
		{
			String str = StdIn.readLine();			
			System.out.println(str + " -> " + map.get(str));
		}	
	}
}
