/*************************************************************************
 * Name        : Keith Vertanen
 * Username    : kvertanen
 * Description : My own version of a hash map that maps a String to a 
 *               String.  Note: this for example purposes only, it 
 *               doesn't use a decent hash function!
 *************************************************************************/

public class StringToStringHash 
{	
	public static final int BUCKETS 	= 256;  // Number of buckets we hash into
	public static final int INIT_SIZE 	= 16;	// Init size of the array at each location
	
	// Inner-class that stores an entry in our hash.
	// We need both the key and value stored so we can handle collisions in a bucket.
	private class Pair
	{
		String key;
		String value;
		
		public Pair(String key, String value)
		{
			this.key = key;
			this.value = value;
		}
	}
	
	// 2D array, the first index is the bucket, the second is holds the (key, value) data 
	private Pair[][] buckets= new Pair[BUCKETS][INIT_SIZE];	
	
	// A very simple (and poor) hash function, sum the char values in the string
	private int hash(String str)
	{
		int result = 0;
		for (int i = 0; i < str.length(); i++)
			result += str.charAt(i);
		return result % BUCKETS;
	}
	
	// Add a new entry to the hash for the (key, value) pair
	public void put(String key, String value)
	{
		int bucket = hash(key);
		for (int i = 0; i < buckets[bucket].length; i++)
		{
			// Either add a new entry at the end of the array or replace existing value
			if ((buckets[bucket][i] == null) || (buckets[bucket][i].key.equals(key)))				
			{
				buckets[bucket][i] = new Pair(key, value);
				return;
			}
		}

		// Double the existing size of the array and copy the data
		int current = buckets[bucket].length;
		Pair [] copy = new Pair[current * 2];
		for (int i = 0; i < current; i++)
			copy[i] = buckets[bucket][i];
		copy[current] = new Pair(key, value);
		buckets[bucket] = copy;
	}
	
	// Given a key, find the value stored in the hash (if any).
	// Return null if we didn't find this key.
	public String get(String key)
	{
		int bucket = hash(key);		
		for (int i = 0; i < buckets[bucket].length; i++)
		{
			// Once we hit a null value, we know we have compared everything
			if (buckets[bucket][i] == null)
				return null;
			
			if (buckets[bucket][i].key.equals(key))
				return buckets[bucket][i].value;
		}
		return null;
	}
	
	// Test main method
	public static void main(String [] args)
	{
		StringToStringHash map = new StringToStringHash();
		
		map.put("www.cnn.com"    , "157.166.255.19" );
		map.put("cs.mtech.edu"   , "150.131.202.136");		
		map.put("www.google.com" , "74.125.224.240" );
		map.put("google.com"     , "74.125.224.240" );
		map.put("www.cnn.com"    , "127.0.0.1" );
		
		// Let the use type a domain name, try and map to an IP
		while (!StdIn.isEmpty())
		{
			String str = StdIn.readLine();			
			System.out.println(str + " -> " + map.get(str));
		}	
	}
}
